package com.softslate.commerce.businessobjects.core;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.EventBus;
import com.google.inject.Injector;
import com.softslate.commerce.customer.core.ActionUtilsImpl;
import com.softslate.commerce.customer.core.FormUtilsImpl;
import com.softslate.commerce.daos.core.DAOFactory;
import com.softslate.commerce.daos.core.DAOUtilsImpl;

/**
 * Factory class used to create instances of business layer objects. Objects are
 * created with the <code>createObject</code>,
 * <code>createObjectFromClassName</code>, or <code>createUtilsObject</code>
 * methods.
 * <p>
 * Generally, this class uses its <code>appComponents</code> property to look up
 * the fully-qualified class name of the requested interface (taken from the
 * <code>appComponents.properties</code> file. Then it uses the Java reflection
 * API to create the instance.
 * <p>
 * An instance of <code>BusinessObjectFactory</code> is created in the Struts
 * layer. Its properties are populated there with the application's
 * {@link com.softslate.commerce.daos.core.DAOFactory},
 * {@link com.softslate.commerce.businessobjects.core.Settings},
 * <code>appComponents</code>, <code>appSettings</code>, and with the
 * {@link com.softslate.commerce.businessobjects.core.User} currently accessing
 * the system.
 * <p>
 * Immediately after instantiating the requested business object, this factory
 * passes its own properties to it so that the newly created object can use them
 * to communicate with the rest of the application.
 * 
 * @author David Tobey
 * 
 */
public class BusinessObjectFactoryImpl extends BaseBusinessObject implements BusinessObjectFactory {

	static Log log = LogFactory.getLog(BusinessObjectFactoryImpl.class);

	public BusinessObjectFactoryImpl() {
		return;
	}

	public BusinessObjectFactoryImpl(Settings settings) {
		setSettings(settings);
		return;
	}

	/**
	 * This factory's <code>User</code>. The current <code>User</code> accessing
	 * the system, for which business processing is to be performed by business
	 * objects created by this factory. This factory will pass the
	 * <code>User</code> to every business object it creates.
	 */
	private User user = null;

	public User getUser() {
		return (this.user);
	}

	public void setUser(User user) {
		this.user = user;
	}

	/**
	 * This factory's <code>Settings</code> instance. The current
	 * <code>Settings</code>, containing various settings to be used by business
	 * objects created by this factory. This factory will pass the
	 * <code>Settings</code> to every business object it creates.
	 */
	private Settings settings = null;

	public Settings getSettings() {
		return (this.settings);
	}

	public void setSettings(Settings settings) {
		this.settings = settings;
	}

	/**
	 * This factory's application settings. The current application settings for
	 * the system. This factory will pass the application settings to every
	 * business object it creates. The application settings are settings for
	 * information unique to a given installation, such as the location of files
	 * on the server, as opposed to the <code>SettingsBean</code>, which stores
	 * settings drawn from the database.
	 */
	private Properties appSettings = null;

	public Properties getAppSettings() {
		return appSettings;
	}

	public void setAppSettings(Properties appSettings) {
		this.appSettings = appSettings;
	}

	/**
	 * This factory's component settings. The component settings identify which
	 * Java classes implement the various interfaces used by the system.
	 */
	private Properties appComponents = null;

	public Properties getAppComponents() {
		return appComponents;
	}

	public void setAppComponents(Properties appComponents) {
		this.appComponents = appComponents;
	}

	/**
	 * This factory's <code>DAOFactory</code>. The current
	 * <code>DAOFactory</code> for the system, which will allow business objects
	 * created by this factory to create data access objects. This factory will
	 * pass the <code>DAOFactory</code> to every business object it creates.
	 */
	private DAOFactory daoFactory = null;

	public DAOFactory getDaoFactory() {
		return (this.daoFactory);
	}

	public void setDaoFactory(DAOFactory daoFactory) {
		this.daoFactory = daoFactory;
	}

	/**
	 * This factory's <code>Injector</code>. When this BusinessObjectFactory
	 * creates and object, it will populate that objects injector property so
	 * the object can use it to invoke other objects using dependency injection
	 * (Guice).
	 */
	private Injector injector = null;

	public Injector getInjector() {
		return injector;
	}

	public void setInjector(Injector injector) {
		this.injector = injector;
	}

	/**
	 * This factory's <code>EventBus</code>. When this BusinessObjectFactory
	 * creates and object, it will populate that object's eventBus property so
	 * the object can use it to post events and trigger subscribers to run their
	 * code.
	 */
	private EventBus eventBus = null;

	public EventBus getEventBus() {
		return eventBus;
	}

	public void setEventBus(EventBus eventBus) {
		this.eventBus = eventBus;
	}

	public <T extends BusinessObject> T createObject(String type) throws Exception {
		String className = getAppComponents().getProperty(type);
		if (className == null) {
			if (log.isWarnEnabled())
				log.warn("Could not find implementing class for " + type);
			return null;
		}
		if (log.isDebugEnabled())
			log.debug("Found implementing class for " + type + ": " + className);
		return (T) createObjectFromClassName(className);
	}

	public <T extends BusinessObject> T createObject(Class<T> type) {
		try {
			if (!type.isInterface()) {
				return (T) createObjectFromClassName(type.getName());
			}
			return (T) createObject(findImplementer(type.getSimpleName()));
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public <T extends BusinessObject> T createObjectFromClassName(String className) throws Exception {
		if (className == null) {
			return null;
		}
		if (log.isDebugEnabled())
			log.debug("Creating class: " + className);
		try {
			Class classDefinition = Class.forName(className);
			return (T) createObjectFromClassDefinition(classDefinition);
		} catch (ClassNotFoundException cnfe) {
			throw new ApplicationException("Class not found in this distribution: " + cnfe.getMessage(), cnfe);
		}
	}

	public <T extends BusinessObject> T createObjectFromClassDefinition(Class classDefinition) throws Exception {
			BusinessObject businessObject = (BusinessObject) classDefinition.newInstance();
			businessObject.initialize(getTransferMap());
		return (T) businessObject;
	}

	public <T> T createUtilsObject(Class<T> type) {
		try {
			return (T) createUtilsObject(findImplementer(type.getSimpleName()));
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public Object createUtilsObject(String type) {
		String className = null;
		try {
			className = getAppComponents().getProperty(type);
			if (log.isDebugEnabled())
				log.debug("Implementing class for " + type + ": " + className);
			Class classDefinition = Class.forName(className);
			return classDefinition.newInstance();
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Could not instantiate class " + className + ": " + e);
			if (type.equals("formUtilsImplementer")) {
				return new FormUtilsImpl();
			} else if (type.equals("businessObjectUtilsImplementer")) {
				return new BusinessObjectUtilsImpl();
			} else if (type.equals("daoUtilsImplementer")) {
				return new DAOUtilsImpl();
			}
			return new ActionUtilsImpl();
		}
	}

	public String findImplementer(String type) {
		if (type.startsWith("Custom") && Character.isUpperCase(type.charAt("Custom".length()))) {
			type = type.substring("Custom".length());
		}
		if (type.endsWith("Impl")) {
			type = type.substring(0, type.length() - "Impl".length());
		}
		if (type.startsWith("SKU")) {
			type = type.substring(0, "SKU".length()).toLowerCase() + type.substring("SKU".length());
		}
		return type.substring(0, 1).toLowerCase() + type.substring(1) + "Implementer";
	}

	public Map invokeMultipleProcessors(String type, String method, Map argument) throws Exception {
		String classNames = getAppComponents().getProperty(type);
		String[] classNameArray = classNames.split(",");
		Map results = new HashMap();
		for (int i = 0; i < classNameArray.length; i++) {
			String className = classNameArray[i].trim();
			if (className.length() == 0) {
				continue;
			}
			try {
				Class classDefinition = Class.forName(className);
				BusinessProcessor bp = (BusinessProcessor) classDefinition.newInstance();
				bp.initialize(getTransferMap());
				Class[] parameterTypes = new Class[] { Map.class };
				Object[] arguments = new Object[] { argument };
				Method m = classDefinition.getMethod(method, parameterTypes);
				Map result = (Map) m.invoke(bp, arguments);
				results.putAll(result);
				if (result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
					if (log.isWarnEnabled())
						log.warn("Method " + method + " of BusinessProcessor " + className
								+ " returned non-sucessful resultCode.");
					return results;
				}
			} catch (InstantiationException e) {
				if (log.isWarnEnabled())
					log.warn("Error instantiating BusinessProcessor class " + className);
				throw e;
			} catch (ClassNotFoundException e) {
				if (log.isWarnEnabled())
					log.warn("Can't find class " + className + "; make sure it's in the app's class path.");
				throw e;
			} catch (ClassCastException e) {
				if (log.isWarnEnabled())
					log.warn("Class " + className + "must implement BusinessProcessor.");
				throw e;
			} catch (NoSuchMethodException e) {
				if (log.isWarnEnabled())
					log.warn("Class " + className + "must implement the " + method + " method.");
				throw e;
			} catch (InvocationTargetException e) {
				if (log.isWarnEnabled())
					log.warn("Error invoking method: " + method, e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw (Exception) e.getCause();
				}
			}
		}
		return results;
	}

	public Collection invokeMultipleCollectionProcessors(String type, String method, Map argument) throws Exception {
		String classNames = getAppComponents().getProperty(type);
		String[] classNameArray = classNames.split(",");
		Collection results = new ArrayList();
		for (int i = 0; i < classNameArray.length; i++) {
			String className = classNameArray[i].trim();
			if (className.length() == 0) {
				continue;
			}
			try {
				Class classDefinition = Class.forName(className);
				BusinessProcessor bp = (BusinessProcessor) classDefinition.newInstance();
				bp.initialize(getTransferMap());
				Class[] parameterTypes = new Class[] { Map.class };
				Object[] arguments = new Object[] { argument };
				Method m = classDefinition.getMethod(method, parameterTypes);
				Collection result = (Collection) m.invoke(bp, arguments);
				if (result != null) {
					results.addAll(result);
				}
			} catch (InstantiationException e) {
				if (log.isWarnEnabled())
					log.warn("Error instantiating BusinessProcessor class " + className, e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw e;
				}
			} catch (ClassNotFoundException e) {
				if (log.isWarnEnabled())
					log.warn("Can't find class " + className + "; make sure it's in the app's class path.", e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw e;
				}
			} catch (ClassCastException e) {
				if (log.isWarnEnabled())
					log.warn("Class " + className + " must implement BusinessProcessor.", e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw e;
				}
			} catch (NoSuchMethodException e) {
				if (log.isWarnEnabled())
					log.warn("Class " + className + "must implement the " + method + " method.", e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw e;
				}
			} catch (InvocationTargetException e) {
				if (log.isWarnEnabled())
					log.warn("Error invoking method: " + method, e);
				if (results.isEmpty() && i == classNameArray.length - 1) {
					throw (Exception) e.getCause();
				}
			}
		}
		return results;
	}

	public Map getTransferMap() {
		Map map = new HashMap();
		map.put("appSettings", getAppSettings());
		map.put("appComponents", getAppComponents());
		map.put("user", getUser());
		map.put("settings", getSettings());
		map.put("daoFactory", getDaoFactory());
		map.put("businessObjectFactory", this);
		map.put("injector", getInjector());
		map.put("eventBus", getEventBus());
		return map;
	}

}
