package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Currency;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TopDocs;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a list of products in the system.
 * <p>
 * <code>SearchListBean</code> is the default implementation of the
 * <code>SearchList</code> interface for the application.
 * 
 * @author Jason McWilliams
 * 
 * @see com.softslate.commerce.businessobjects.product.SearchList
 */
public class SearchListBean extends BaseBusinessObject implements SearchList,
		Serializable {

	private static final long serialVersionUID = 387769791810353922L;

	public static final int SEARCH_PRODUCTS = 1;

	public static final int SEARCH_CATEGORIES = 2;

	public static final int SEARCH_MANUFACTURERS = 3;
	
	public static final int SEARCH_ALL = 4;

	public static final int SEARCH_PRODUCTS_WITHIN_CATEGORY = 5;

	public static final int SEARCH_PRODUCTS_WITHIN_MANUFACTURER = 6;

	public static final int SEARCH_PRODUCTS_FOR_PRODUCT_LIST = 7;

	public static final int PRODUCT = 1;

	public static final int CATEGORY = 2;

	public static final int MANUFACTURER = 3;

	static Log log = LogFactory.getLog(SearchListBean.class);

	private int[] objectTypes;

	public int[] getObjectTypes() {
		return objectTypes;
	}

	public void setObjectTypes(int[] objectTypes) {
		this.objectTypes = objectTypes;
	}

	private float[] objectScores;

	public float[] getObjectScores() {
		return objectScores;
	}

	public void setObjectScores(float[] objectScores) {
		this.objectScores = objectScores;
	}

	private String[] objectExcerpts;

	public String[] getObjectExcerpts() {
		return objectExcerpts;
	}

	public void setObjectExcerpts(String[] objectExcerpts) {
		this.objectExcerpts = objectExcerpts;
	}

	private int searchListType = 1;

	public int getSearchListType() {
		return (this.searchListType);
	}

	public void setSearchListType(int searchListType) {
		this.searchListType = searchListType;
	}

	private int categoryID = 0;

	public int getCategoryID() {
		return (this.categoryID);
	}

	public void setCategoryID(int categoryID) {
		this.categoryID = categoryID;
	}

	private int manufacturerID = 0;

	public int getManufacturerID() {
		return manufacturerID;
	}

	public void setManufacturerID(int manufacturerID) {
		this.manufacturerID = manufacturerID;
	}

	private int itemsPerPage = 10;

	public int getItemsPerPage() {
		return (this.itemsPerPage);
	}

	public void setItemsPerPage(int itemsPerPage) {
		this.itemsPerPage = itemsPerPage;
	}

	private int firstRow = 1;

	public int getFirstRow() {
		return (this.firstRow);
	}

	public void setFirstRow(int firstRow) {
		this.firstRow = firstRow;
	}

	private int itemCount = 0;

	public int getItemCount() {
		return (this.itemCount);
	}

	public void setItemCount(int itemCount) {
		this.itemCount = itemCount;
	}

	private String searchString = null;

	public String getSearchString() {
		return (this.searchString);
	}

	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}

	private String sortBy = "score";

	public String getSortBy() {
		return (this.sortBy);
	}

	public void setSortBy(String sortBy) {
		this.sortBy = sortBy;
	}

	private String indexDir = null;

	public String getIndexDir() {
		return indexDir;
	}

	public void setIndexDir(String indexDir) {
		this.indexDir = indexDir;
	}

	private Collection searchListCollection = null;

	public Collection getSearchListCollection() {
		return (this.searchListCollection);
	}

	public void setSearchListCollection(Collection searchListCollection) {
		this.searchListCollection = searchListCollection;
	}

	private Collection luceneDocCollection = null;

	public Collection getLuceneDocCollection() {
		return (this.luceneDocCollection);
	}

	public void setLuceneDocCollection(Collection luceneDocCollection) {
		this.luceneDocCollection = luceneDocCollection;
	}

	private Collection facetResultCollection = null;

	public Collection getFacetResultCollection() {
		return (this.facetResultCollection);
	}

	public void setFacetResultCollection(Collection facetResultCollection) {
		this.facetResultCollection = facetResultCollection;
	}

	private Collection simpleFacetResultCollection = null;

	public Collection getSimpleFacetResultCollection() {
		return (this.simpleFacetResultCollection);
	}

	public void setSimpleFacetResultCollection(Collection simpleFacetResultCollection) {
		this.simpleFacetResultCollection = simpleFacetResultCollection;
	}

	private Map simpleFacetResultMap = null;

	public Map getSimpleFacetResultMap() {
		return (this.simpleFacetResultMap);
	}

	public void setSimpleFacetResultMap(Map simpleFacetResultMap) {
		this.simpleFacetResultMap = simpleFacetResultMap;
	}

	private TopDocs topDocs = null;

	public TopDocs getTopDocs() {
		return (this.topDocs);
	}

	public void setTopDocs(TopDocs topDocs) {
		this.topDocs = topDocs;
	}

	private Query luceneQuery = null;

	public Query getLuceneQuery() {
		return luceneQuery;
	}

	public void setLuceneQuery(Query luceneQuery) {
		this.luceneQuery = luceneQuery;
	}
	
	private String[] facets;
	
	public String[] getFacets() {
		return facets;
	}

	public void setFacets(String[] facets) {
		this.facets = facets;
	}
	
	private Map facetCategories;
	
	public Map getFacetCategories() {
		return facetCategories;
	}

	public void setFacetCategories(Map facetCategories) {
		this.facetCategories = facetCategories;
	}

	private Map facetManufacturers;

	public Map getFacetManufacturers() {
		return facetManufacturers;
	}

	public void setFacetManufacturers(Map facetManufacturers) {
		this.facetManufacturers = facetManufacturers;
	}

	private Map categoryMap;
	
	public Map getCategoryMap() {
		return categoryMap;
	}

	public void setCategoryMap(Map categoryMap) {
		this.categoryMap = categoryMap;
	}

	public void addFormatting(Locale locale, String currencyCode) {
		if (currencyCode == null) {
			currencyCode = "USD";
		}
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		Iterator rows = getSearchListCollection().iterator();
		while (rows.hasNext()) {
			Object object = rows.next();
			if (object != null && object.getClass() == Product.class) {
				Product product = (Product) object;
				product.addFormatting(locale, currencyCode);
			}
		}
	}
}
