package com.softslate.commerce.customer.product;

import java.util.Collection;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionServlet;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.businessobjects.product.ProductProcessor;

/**
 * Initializes singleton objects related to products, in the application scope.
 * Called by {@link com.softslate.commerce.customer.core.BaseRequestProcessor}
 * <!-- --> at the beginning of a request, if the singletons are found to be
 * missing.
 * 
 * @author David Tobey
 * 
 */
public class ProductInitializer {

	static Log log = LogFactory.getLog(ProductInitializer.class);

	public ProductInitializer() {
	}

	public ProductInitializer(ActionServlet servlet) {
		setServlet(servlet);
	}

	private ActionServlet servlet = null;

	public ActionServlet getServlet() {
		return (this.servlet);
	}

	public void setServlet(ActionServlet servlet) {
		this.servlet = servlet;
	}

	/**
	 * Creates a <code>Collection</code> representing the tree of categories in
	 * the system. Places the <code>Collection</code> in the application scope
	 * as an attribute with the name "categoryTree" so it can be referenced on
	 * any store screen. (Use <code>application.categoryTree</code> to reference
	 * the <code>Collection</code> from within a JSP template.) Each item in the
	 * <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Category}, which
	 * corresponds to each one of the "top level" categories defined in the
	 * store (i.e., categories whose <code>parentCategory</code> property is
	 * <code>null</code>). To render the tree, iterate through the top level
	 * categories and recursively navigate through each of their
	 * <code>subcategoryCollection</code>s in turn.
	 * <p>
	 * This method invokes
	 * {@link com.softslate.commerce.businessobjects.product.ProductProcessor#loadCategoryTree()}
	 * to load the category information from the database.
	 * 
	 * @param businessObjectFactory
	 *            An initialized <code>BusinessObjectFactory</code> used to
	 *            create the <code>ProductProcessor</code> implementer that
	 *            loads the category information from the database.
	 */
	public void initializeCategoryTree(
			BusinessObjectFactory businessObjectFactory) {
		if (log.isInfoEnabled())
			log.info("Initializing category tree.");

		// Skip if the initializeCategoryTreeAndMap setting is off
		Settings settings = (Settings) servlet.getServletContext().getAttribute("settings");
		String initializeCategoryTreeAndMap = settings.getValue("initializeCategoryTreeAndMap");
		if ("0".equals(initializeCategoryTreeAndMap)) {
			return;
		}

		// Skip if we have not upgraded to at least 3.1.1, where a new category
		// field is defined
		String dataObjectsVersion = (String) settings
				.getValue("dataObjectsVersion");
		if (dataObjectsVersion.startsWith("2.")
				|| dataObjectsVersion.startsWith("3.0")
				|| dataObjectsVersion.startsWith("3.1.0")) {
			return;
		}
		try {
			synchronized (getServlet().getServletContext()) {
				ProductProcessor productProcessor =  businessObjectFactory
						.createObject(ProductProcessor.class);
				Collection categoryTree = productProcessor.loadCategoryTree();
				Map categoryMap = productProcessor.createCategoryMapFromCategoryTree(categoryTree);
				getServlet().getServletContext().setAttribute("categoryTree",
						categoryTree);
				getServlet().getServletContext().setAttribute("categoryMap",
						categoryMap);
				getServlet().getServletContext().setAttribute("categoryTreeRebuilt", Boolean.TRUE);
			}
		} catch (Exception e) {
			log.error("Couldn't build category tree: " + e.toString(), e);
		}
		return;
	}

	/**
	 * Creates a <code>Collection</code> containing each of the built-in
	 * categories in the system. Built-in categories are typically composed of
	 * specials or other featured products that need to displayed on arbitrary
	 * pages thoughout the store. This method places the <code>Collection</code>
	 * in the application scope as an attribute with the name
	 * "builtInCategories" so it can be referenced on any store screen. Each
	 * item in the <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Category}. (Use
	 * <code>application.builtInCategories</code> to reference the
	 * <code>Collection</code> from within a JSP template.)
	 * <p>
	 * Built-in categories are definined by the "builtInCategories" setting in
	 * the <code>sscSetting</code> database table, a comma-separated list of
	 * category codes.
	 * <p>
	 * This method invokes
	 * {@link com.softslate.commerce.businessobjects.product.ProductProcessor#loadBuiltInCategories()}
	 * to load the category information from the database.
	 * 
	 * @param businessObjectFactory
	 *            An initialized <code>BusinessObjectFactory</code> used to
	 *            create the <code>ProductProcessor</code> implementer that
	 *            loads the category information from the database.
	 */
	public void initializeBuiltInCategories(
			BusinessObjectFactory businessObjectFactory) {
		if (log.isInfoEnabled())
			log.info("Initializing built in categories.");

		// Skip if we have not upgraded to at least 3.1.1, where a new category
		// field is defined
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");
		String dataObjectsVersion = (String) settings
				.getValue("dataObjectsVersion");
		if (dataObjectsVersion.startsWith("2.")
				|| dataObjectsVersion.startsWith("3.0")
				|| dataObjectsVersion.startsWith("3.1.0")) {
			return;
		}
		try {
			synchronized (getServlet().getServletContext()) {
				ProductProcessor productProcessor =  businessObjectFactory
						.createObject(ProductProcessor.class);

				// Load the built in categories (featured in the sidebar)
				Collection builtInCategories = productProcessor
						.loadBuiltInCategories();
				if (log.isDebugEnabled())
					log.debug("builtInCategories: " + builtInCategories);
				getServlet().getServletContext().setAttribute(
						"builtInCategories", builtInCategories);

				// Load the header categories (appear as links in the header)
				String headerCategoriesAndPagesList = (String) settings
						.getValue("headerCategoriesAndPages");
				Collection headerCategoriesAndPages = productProcessor
						.loadCategories(headerCategoriesAndPagesList);
				if (log.isDebugEnabled())
					log.debug("headerCategoriesAndPages: "
							+ headerCategoriesAndPages);
				getServlet().getServletContext().setAttribute(
						"headerCategoriesAndPages", headerCategoriesAndPages);

				// Load the footer categories (appear as links in the
				// footer)
				String footerCategoriesAndPagesList = (String) settings
						.getValue("footerCategoriesAndPages");
				Collection footerCategoriesAndPages = productProcessor
						.loadCategories(footerCategoriesAndPagesList);
				if (log.isDebugEnabled())
					log.debug("footerCategoriesAndPages: "
							+ footerCategoriesAndPages);
				getServlet().getServletContext().setAttribute(
						"footerCategoriesAndPages", footerCategoriesAndPages);
			}
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Couldn't load built in categories: " + e.toString());
		}
		return;
	}

	/**
	 * Creates a <code>Collection</code> containing each of the manufacturers in
	 * the system. This method places the <code>Collection</code> in the
	 * application scope as an attribute with the name "manufacturers" so it can
	 * be referenced on any store screen. Each item in the
	 * <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Manufacturer}. (Use
	 * <code>application.manufacturers</code> to reference the
	 * <code>Collection</code> from within a JSP template.)
	 * <p>
	 * This method invokes
	 * {@link com.softslate.commerce.businessobjects.product.ManufacturerProcessor#loadActiveManufacturers()}
	 * to load the manufacturers' information from the database.
	 * 
	 * @param businessObjectFactory
	 *            An initialized <code>BusinessObjectFactory</code> used to
	 *            create the <code>ManufacturerProcessor</code> implementer that
	 *            loads the manufacturers' information from the database.
	 */
	public void initializeManufacturers(
			BusinessObjectFactory businessObjectFactory) {
		if (log.isInfoEnabled())
			log.info("Initializing manufacturers.");

		// Skip if we have not upgraded to at least 3.0.0, where new
		// manufacturer fields are defined.
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");
		String dataObjectsVersion = (String) settings
				.getValue("dataObjectsVersion");
		if (dataObjectsVersion.startsWith("2.")) {
			return;
		}
		try {
			synchronized (getServlet().getServletContext()) {
				ManufacturerProcessor manufacturerProcessor =  businessObjectFactory
						.createObject(ManufacturerProcessor.class);
				Collection manufacturers = manufacturerProcessor
						.loadActiveManufacturers();
				if (log.isDebugEnabled())
					log.debug("manufacturers: " + manufacturers);
				getServlet().getServletContext().setAttribute("manufacturers",
						manufacturers);
			}
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Couldn't load manufacturers: " + e.toString());
		}
		return;
	}

	/**
	 * Creates a <code>Collection</code> containing discounts for display within
	 * the store. The discounts include any global discount (which apply to the
	 * user's cart as a whole rather than to particular line items), or
	 * discounts where a condition of the discount is that a given product is in
	 * the user's cart (querying for these discounts here allows us to display
	 * them alongside the required products, as a method of upselling). This
	 * method places the <code>Collection</code> in the application scope as an
	 * attribute with the name "storeDiscounts" so it can be referenced on any
	 * store screen. Each item in the <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Discount}. (Use
	 * <code>application.storeDiscounts</code> to reference the
	 * <code>Collection</code> from within a JSP template.)
	 * <p>
	 * This method invokes
	 * {@link com.softslate.commerce.businessobjects.product.DiscountProcessor#loadStoreDiscounts()}
	 * to load the discounts' information from the database.
	 * 
	 * @param businessObjectFactory
	 *            An initialized <code>BusinessObjectFactory</code> used to
	 *            create the <code>DiscountProcessor</code> implementer that
	 *            loads the discounts' information from the database.
	 */
	public void initializeStoreDiscounts(
			BusinessObjectFactory businessObjectFactory) {
		if (log.isInfoEnabled())
			log.info("Initializing discounts.");

		// Skip if we have not upgraded to at least 3.1.8, where a new discount
		// field is defined
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");
		String applyDiscounts = (String) settings.getValue("applyDiscounts");
		if (applyDiscounts == null || applyDiscounts.equals("0")) {
			return;
		}
		String dataObjectsVersion = (String) settings
				.getValue("dataObjectsVersion");
		if (dataObjectsVersion == null || dataObjectsVersion.startsWith("2.")
				|| dataObjectsVersion.startsWith("3.0")
				|| dataObjectsVersion.startsWith("3.1.0")
				|| dataObjectsVersion.startsWith("3.1.1")
				|| dataObjectsVersion.startsWith("3.1.2")
				|| dataObjectsVersion.startsWith("3.1.3")
				|| dataObjectsVersion.startsWith("3.1.4")
				|| dataObjectsVersion.startsWith("3.1.5")
				|| dataObjectsVersion.startsWith("3.1.6")
				|| dataObjectsVersion.startsWith("3.1.7")) {
			return;
		}
		try {
			synchronized (getServlet().getServletContext()) {
				DiscountProcessor discountProcessor =  businessObjectFactory
						.createObject(DiscountProcessor.class);
				Collection storeDiscounts = discountProcessor
						.loadStoreDiscounts();
				if (log.isDebugEnabled())
					log.debug("storeDiscounts: " + storeDiscounts);
				getServlet().getServletContext().setAttribute("storeDiscounts",
						storeDiscounts);
			}
		} catch (Exception e) {
			if (log.isWarnEnabled())
				log.warn("Couldn't load manufacturers: " + e.toString());
		}
		return;
	}

}
