package com.softslate.commerce.daos.core;

import java.util.Properties;

import javax.sql.DataSource;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.core.Settings;

/**
 * @author David Tobey
 */
public interface DAOFactory {

    /**
     * Used in 1.x with
     * com.softslate.commerce.daos.core.LegacyDAOFactory, which is
     * replaced in 2.x with the
     * {@link com.softslate.commerce.daos.core.HibernateDAOFactory}. Also used
     * in the 1.x to 2.x upgrade process (which is why it is not deprecated).
     */
    abstract DataSource getJndiDataSource() throws Exception;


    abstract String getDatabaseType();

    abstract void setDatabaseType(String databaseType);

    /**
     * Retrieves this factory's <code>Settings</code> instance. The current
     * <code>Settings</code>, containing various settings to be used by DAOs
     * created by this factory. This factory will pass the <code>Settings</code>
     * to every object it creates.
     */
    public Settings getSettings();

    public void setSettings(Settings settings);

    /**
     * Retrieves this factory's component settings. The component settings
     * identify which Java class implement the various Interfaces used by the
     * system.
     */
    public Properties getAppComponents();

    public void setAppComponents(Properties appComponents);

    /**
     * Retrieves this factory's default component settings. The component settings
     * identify which Java class implement the various Interfaces used by the
     * system.
     */
    public Properties getAppComponentsDefaults();

    public void setAppComponentsDefaults(Properties appComponentsDefaults);

    /**
     * Retrieves this factory's application settings; the current application
     * settings for the system. This factory will pass the application settings
     * to every DAO it creates. The application settings are settings for
     * information unique to a given installation, such as the location of files
     * on the server, as opposed to the <code>Settings</code> instance, which
     * stores settings drawn from the database.
     */
    public Properties getAppSettings();

    public void setAppSettings(Properties appSettings);

    /**
     * Retrieves this factory's Hibernate settings. The Hibernate settings are
     * explicitly set for the HibernateDAOFactory before it is initialized, so
     * that they can be changed after the installer tool is run.
     */
    public Properties getHibernateSettings();

    public void setHibernateSettings(Properties hibernateSettings);

	/**
	 * Retrieves the <code>Injector</code> previously set for this factory.
	 * 
	 * @return A <code>Injector</code> object that can be used to create other
	 *         objects and inject them with dependencies.
	 */
	public Injector getInjector();

	/**
	 * Sets this factory's <code>Injector</code>. This factory will pass the
	 * <code>Injector</code> to every business object it creates.
	 * 
	 * @param injector
	 *            The current <code>Injector</code> for the system, which can be
	 *            used to create other objects and inject them with
	 *            dependencies.
	 */
	public void setInjector(Injector injector);

    /**
     * Retrieves this factory's Hibernate <code>SessionFactory</code>.
     */
    public SessionFactory getSessionFactory();

    public void setSessionFactory(SessionFactory sessionFactory);

    /**
     * Initializes this <code>DAOFactory</code>. Called by
     * <code>BaseRequestProcessor</code>.
     * 
     * @throws Exception
     */
    abstract void initialize() throws Exception;

    /**
     * Creates and initializes a data access object for use by the application.
     * Uses the <code>appSettings</code> property to look up the
     * fully-qualified class name of the requested data access object.
     * <p>
     * Immediately after instantiating the requested DAO, this factory passes
     * its own properties to it so that the newly created object can use them to
     * communicate with the rest of the application.
     * 
     * @param type
     *            A <code>String</code> associated with the class name of the
     *            data access object to be created.
     * @return An instantiated and initialized data access object, or
     *         <code>null</code> if initialization failed.
     * @throws Exception
     */
    public Object createDAO(String type) throws Exception;

    /**
     * Creates and initializes a data access object for use by the application
     * using the incoming class name.
     * <p>
     * Immediately after instantiating the requested DAO, this factory passes
     * its own properties to it so that the newly created object can use them to
     * communicate with the rest of the application.
     * 
     * @param className
     *            A <code>String</code> representing the class name of the
     *            data access object to be created.
     * @return An instantiated and initialized data access object, or
     *         <code>null</code> if initialization failed.
     * @throws Exception
     */
    public Object createDAOFromClassName(String className) throws Exception;

	public <T> T createDAO(Class<T> type);

    public Session startSession();

    public Session getSession();

    public void closeSession();

    public void beginTransaction();

    public void commitTransaction() throws HibernateException;

    public void rollbackTransactionKeepingSessionOpen();
    
    public void rollbackTransaction();
    
    public void evictCache();
}
