package com.softslate.commerce.daos.core;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.RequestLog;
import com.softslate.commerce.businessobjects.core.RequestLogProcessor;

/**
 * Generic DAO that runs miscellaneous queries related to the request log.
 * 
 * @author David Tobey
 */
public class RequestLogGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements RequestLogGatewayDAO {

	static Log log = LogFactory.getLog(RequestLogGatewayDAOHibernate.class);

	public Map loadRequestLogsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteRequestLogs(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Collection loadDistinctSessionIDsFromCustomerID(int parentCustomerID) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "requestLog.distinctSessionIDsFromCustomerID");
			q.setInteger("customerID", parentCustomerID);
			return q.list();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadDistinctSessionIDsFromOrderID(int parentOrderID) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "requestLog.distinctSessionIDsFromOrderID");
			q.setInteger("orderID", parentOrderID);
			return q.list();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {
		parameters.put("parentID", null);
		crit = super.processCommonCriteria(crit, parameters);
		String workWithRequests = (String) parameters.get("workWithRequests");
		if (workWithRequests == null || workWithRequests.equals("Customer")) {
			crit.add(Restrictions.eq("isAdminRequest", Boolean.FALSE));
			Collection sessionIDs = (Collection) parameters.get("sessionIDs");
			if (sessionIDs != null) {
				crit.add(Restrictions.in("sessionID", sessionIDs));
			}
		} else if (workWithRequests.equals("Administrator")) {
			crit.add(Restrictions.eq("isAdminRequest", Boolean.TRUE));
			String parentOrderID = (String) parameters.get("parentOrderID");
			if (parentOrderID != null && parentOrderID.length() > 0) {
				crit.add(Restrictions.eq("orderID", new Integer(parentOrderID)));
			}
			String parentCustomerID = (String) parameters.get("parentCustomerID");
			if (parentCustomerID != null && parentCustomerID.length() > 0) {
				crit.add(Restrictions.eq("customerID", new Integer(parentCustomerID)));
			}
		}
		return crit;
	}

	public Collection loadRequestLogStatistics() throws DataAccessException {
		Collection stats = new ArrayList();
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "requestLog.selectRecentRequestLogCount");
			// Last 24 hours
			Calendar start = Calendar.getInstance();
			start.add(Calendar.HOUR_OF_DAY, -24);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 7 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -7);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 30 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -30);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 90 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -90);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// All time
			Query q2 = getNamedQuery(session, "requestLog.selectAllRequestLogCount");
			stats.add(q2.uniqueResult());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return stats;
	}

	public int pruneRequestLog(boolean commit, Map parameters) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "requestLog.pruneRequestLog");
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, (-1 * Integer.parseInt((String) parameters.get("olderThanDays"))));
			start.add(Calendar.HOUR_OF_DAY, (-1 * Integer.parseInt((String) parameters.get("olderThanHours"))));
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			int updateCount = q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			return updateCount;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		RequestLogProcessor processor =  bof.createObject(RequestLogProcessor.class);
		Map results = processor.postChangingRequestLogEvent((RequestLog) item, parameters);
		return results;
	}
}
