package com.softslate.commerce.daos.core;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.StringType;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Setting;
import com.softslate.commerce.businessobjects.core.Settings;

/**
 * Generic DAO for setting and retrieving settings from the database.
 * 
 * @author David Tobey
 * 
 */
public class SettingsDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements SettingsDAO {

	static Log log = LogFactory.getLog(SettingsDAOHibernate.class);

	private Setting setting = null;

	public Setting getSetting() {
		return (this.setting);
	}

	public void setSetting(Setting setting) {
		this.setting = setting;
	}

	private Settings settings = null;

	public Settings getSettings() {
		return (this.settings);
	}

	public void setSettings(Settings settings) {
		this.settings = settings;
	}

	public Settings loadSettings(Settings settings) throws Exception {
		setSettings(settings);
		List results = (List) loadSettings();
		getSettings().setMap(generateSettingsMap(results));
		return getSettings();
	}

	public Collection loadSettings() throws DataAccessException {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "setting.selectAll");
			results = q.list();

			// Decrypt the encrypted settings
			getDaoFactory().getSession().clear();
			Iterator i = results.iterator();
			while (i.hasNext()) {
				Setting setting = (Setting) i.next();
				if (getAppSettings() != null && setting.getValueType() != null
						&& setting.getValueType().equals("encryptedValue")) {
					String settingValue = setting.getMediumValue();
					settingValue = getDAOUtils().getDecrypted(settingValue,
							"twoWay",
							getAppSettings().getProperty("twoWayKeyFile"));
					setting.setMediumValue(settingValue);
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return results;
	}

	@Override
	public Map loadSettingsByType(String settingType) throws Exception {
		Map settingsMap = new HashMap();
		List results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "setting.selectByType");
			q.setString("settingType", settingType);
			results = q.list();

			// Decrypt the encrypted settings
			Iterator i = results.iterator();
			while (i.hasNext()) {
				Setting setting = (Setting) i.next();
				if (getAppSettings() != null && setting.getValueType() != null
						&& setting.getValueType().equals("encryptedValue")) {
					String settingValue = setting.getMediumValue();
					settingValue = getDAOUtils().getDecrypted(settingValue, "twoWay",
							getAppSettings().getProperty("twoWayKeyFile"));
					setting.setMediumValue(settingValue);
				}
			}
			settingsMap = this.generateSettingsMap(results);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return settingsMap;
	}

	public Settings updateSettings(Map newSettings, Settings settings,
			Setting setting) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to update settings.");
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "setting.selectFromCodes");
			q.setParameterList("codes", newSettings.keySet(), StringType.INSTANCE);
			results = new LinkedHashSet(q.list());
			Iterator i = results.iterator();
			while (i.hasNext()) {
				setSetting((Setting) i.next());
				String newValue = (String) newSettings.get(getSetting()
						.getCode());
				if (getSetting().getValueType().equals("encryptedValue") && newValue != null) {
					newValue = getDAOUtils().getEncrypted(newValue, "twoWay",
							getAppSettings().getProperty("twoWayKeyFile"), "sha512");
				} else if (log.isDebugEnabled()) {
					log.debug("newValue: " + newValue);
				}
				Map parameters = new HashMap();
				parameters.put("newValue", newValue);
				if (getSetting().getValueType().equals("booleanValue")
						|| getSetting().getValueType().equals("integerValue")) {
					parameters.put("integerValue", newValue);
				} else if (getSetting().getValueType().equals("smallValue")) {
					parameters.put("smallValue", newValue);
				} else if (getSetting().getValueType().equals("mediumValue")
						|| getSetting().getValueType().equals("encryptedValue")) {
					parameters.put("mediumValue", newValue);
				} else if (getSetting().getValueType().equals("textValue")) {
					parameters.put("textValue", newValue);
				}
				settings.postChangingSettingEvent(getSetting(), parameters);
				if (getSetting().getValueType().equals("booleanValue")
						|| getSetting().getValueType().equals("integerValue")) {
					getSetting().setIntegerValue(new Integer(newValue));
				} else if (getSetting().getValueType().equals("smallValue")) {
					getSetting().setSmallValue(newValue);
				} else if (getSetting().getValueType().equals("mediumValue")
						|| getSetting().getValueType().equals("encryptedValue")) {
					getSetting().setMediumValue(newValue);
				} else if (getSetting().getValueType().equals("textValue")) {
					getSetting().setTextValue(newValue);
				} else {
					if (log.isWarnEnabled())
						log.warn("Value type not recognized: "
								+ getSetting().getValueType() + " newValue: "
								+ newValue);
				}
			}
			settings = loadSettings(settings);
			getDaoFactory().commitTransaction();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}

		return settings;
	}

	public Settings insertSettings(Map newSettings, Settings settings, Setting setting) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to insert settings.");

		// Iterate through newSettings, check to see if the setting already exists in settings, and insert if it does
		// not already exist.
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "setting.selectFromCodes");
			q.setParameterList("codes", newSettings.keySet(), StringType.INSTANCE);
			results = new LinkedHashSet(q.list());
			Iterator i = results.iterator();
			while (i.hasNext()) {
				setSetting((Setting) i.next());
				String newValue = (String) newSettings.get(getSetting().getCode());
				if (getSetting().getValueType().equals("encryptedValue") && newValue != null) {
					newValue = getDAOUtils().getEncrypted(newValue, "twoWay",
							getAppSettings().getProperty("twoWayKeyFile"), "sha512");
				} else if (log.isDebugEnabled()) {
					log.debug("newValue: " + newValue);
				}
				Map parameters = new HashMap();
				parameters.put("newValue", newValue);
				if (getSetting().getValueType().equals("booleanValue")
						|| getSetting().getValueType().equals("integerValue")) {
					parameters.put("integerValue", newValue);
				} else if (getSetting().getValueType().equals("smallValue")) {
					parameters.put("smallValue", newValue);
				} else if (getSetting().getValueType().equals("mediumValue")
						|| getSetting().getValueType().equals("encryptedValue")) {
					parameters.put("mediumValue", newValue);
				} else if (getSetting().getValueType().equals("textValue")) {
					parameters.put("textValue", newValue);
				}
				settings.postChangingSettingEvent(getSetting(), parameters);
				if (getSetting().getValueType().equals("booleanValue")
						|| getSetting().getValueType().equals("integerValue")) {
					getSetting().setIntegerValue(new Integer(newValue));
				} else if (getSetting().getValueType().equals("smallValue")) {
					getSetting().setSmallValue(newValue);
				} else if (getSetting().getValueType().equals("mediumValue")
						|| getSetting().getValueType().equals("encryptedValue")) {
					getSetting().setMediumValue(newValue);
				} else if (getSetting().getValueType().equals("textValue")) {
					getSetting().setTextValue(newValue);
				} else {
					if (log.isWarnEnabled())
						log.warn(
								"Value type not recognized: " + getSetting().getValueType() + " newValue: " + newValue);
				}
			}
			settings = loadSettings(settings);
			getDaoFactory().commitTransaction();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}

		return settings;
	}

	public HashMap generateSettingsMap(List results) throws Exception {
		Iterator i = results.iterator();
		HashMap map = new HashMap(results.size());
		while (i.hasNext()) {
			Setting setting = (Setting) i.next();
			String settingCode = setting.getCode();
			String settingValueType = setting.getValueType();
			String settingValue;
			if (settingValueType.equals("encryptedValue")) {
				settingValue = setting.getMediumValue();
			} else if (settingValueType.equals("smallValue")) {
				settingValue = setting.getSmallValue();
			} else if (settingValueType.equals("mediumValue")) {
				settingValue = setting.getMediumValue();
			} else if (settingValueType.equals("textValue")) {
				settingValue = setting.getTextValue();
			} else if ((settingValueType.equals("booleanValue") || settingValueType
					.equals("integerValue"))
					&& setting.getIntegerValue() != null) {
				settingValue = setting.getIntegerValue().toString();
			} else {
				settingValue = "0";
			}
			if (settingValue == null) {
				settingValue = "";
			}
			map.put(settingCode, settingValue);
		}
		return map;
	}

	public void insertSetting() throws DataAccessException {
		insertSetting(false);
	}

	public void insertSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(getSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadSettingFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Setting record = (Setting) session.get(Setting.class, new Integer(
					getSetting().getSettingID()));
			PropertyUtils.copyProperties(getSetting(), record);
			setSetting(record);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateSetting() throws DataAccessException {
		updateSetting(false);
	}

	public void updateSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteSetting() throws DataAccessException {
		deleteSetting(false);
	}

	public void deleteSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setSetting((Setting) session.load(Setting.class, new Integer(
					getSetting().getSettingID())));
			session.delete(getSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
		return;
	}

	public Map loadSettingsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {
		crit = super.processCommonCriteria(crit, parameters);
		crit.add(Restrictions.eq("type", "USERDEFINED"));
		return crit;
	}

	public Map editDeleteUserSettings(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		Settings settingsBean =  bof.createObject(Settings.class);
		Map results = settingsBean.postChangingSettingEvent((Setting) item, parameters);
		return results;
	}

}
